SystemOrganization addCategory: #'CT-Updater'!

!TheWorldMainDockingBar methodsFor: '*ct-updater' stamp: 'mt 4/9/2010 11:00:18.238'!
extendSqueakMenu: aMenu
	"Insert custom entries in the Squeak menu of the world main docking bar. Is CodeTalk aware."
	
	(self respondsTo: #updateProjectTalkMenuItemOn:)
		ifTrue: [self updateProjectTalkMenuItemOn: aMenu].
	
	self updateCodeTalkMenuItemOn: aMenu.
	
	aMenu	addLine.! !

!TheWorldMainDockingBar methodsFor: '*ct-updater-override' stamp: 'mt 4/9/2010 10:59:35.3'!
squeakMenuOn: aDockingBar 
	"Private - fill the given docking bar"
	
	aDockingBar addItem: [ :item |
		item
			contents: '';
			icon: MenuIcons squeakLogoIcon;
			selectedIcon: MenuIcons squeakLogoInvertedIcon;
			addSubMenu: [ :menu | 
				self
					aboutMenuItemOn: menu;
					updateMenuItemOn: menu.
				menu addLine.
				self extendSqueakMenu: menu.
				self 
					saveMenuItemOn: menu;
					saveAsMenuItemOn: menu;
					saveAsNewVersionMenuItemOn: menu.
				menu addLine.
				self
					saveAndQuitMenuItemOn: menu;
					quitMenuItemOn: menu ] ]! !

!TheWorldMainDockingBar methodsFor: '*ct-updater' stamp: 'mt 4/9/2010 11:00:37.343'!
updateCodeTalk
	"Updates CodeTalk."
	
	CtUpdater update.! !

!TheWorldMainDockingBar methodsFor: '*ct-updater' stamp: 'mt 4/9/2010 11:00:55.398'!
updateCodeTalkMenuItemOn: menu

	menu addItem: [ :item |
		item
			contents: 'Update CodeTalk' translated;
			help: 'Load latest code updates via the internet' translated;
			target: self;
			selector: #updateCodeTalk ]! !

Object subclass: #CtUpdateTask
	instanceVariableNames: 'package newerVersionString'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CT-Updater'!

!CtUpdateTask class methodsFor: 'instance creation' stamp: 'mt 4/9/2010 09:35:16.37'!
package: aMCPackage

	^ self new
		package: aMCPackage! !

!CtUpdateTask methodsFor: 'info' stamp: 'mt 4/9/2010 10:26:16.909'!
currentVersionString

	self package
		ifNil: [^ '(No package.)'].
	self package workingCopy ancestry ancestors
		ifEmpty: [^ '(No version info.)'].
		
	^ self package workingCopy ancestry ancestors first name! !

!CtUpdateTask methodsFor: 'accessing' stamp: 'mt 4/9/2010 09:54:36.109'!
newerVersionString
	^ newerVersionString! !

!CtUpdateTask methodsFor: 'accessing' stamp: 'mt 4/9/2010 09:54:36.205'!
newerVersionString: anObject
	newerVersionString := anObject! !

!CtUpdateTask methodsFor: 'accessing' stamp: 'mt 4/9/2010 09:34:47.283'!
package
	^ package! !

!CtUpdateTask methodsFor: 'accessing' stamp: 'mt 4/9/2010 09:57:49.257'!
package: anObject

	package := anObject.
	self setNewerVersionString.! !

!CtUpdateTask methodsFor: 'accessing' stamp: 'mt 4/9/2010 10:34:48.271'!
setNewerVersionString

	| pnv maxVer nvn |
	self package
		ifNil: [self newerVersionString: ''. ^ self].	
			
	pnv := self package workingCopy possiblyNewerVersions.
	
	pnv ifEmpty: [self newerVersionString: ''. ^ self].
	
	maxVer := (pnv collect: [:name | (name findTokens: {$.}) last asNumber]) max.
	nvn := (pnv detect: [:v | v endsWith: maxVer asString]).
	
	self currentVersionString = nvn
		ifTrue: [self newerVersionString: ''. ^self].
		
	self newerVersionString: nvn.! !

!CtUpdateTask methodsFor: 'updating' stamp: 'mt 4/9/2010 10:02:00.66'!
update

	| repo |
	repo := self package workingCopy repositoryGroup repositories
		detect: [:r | r includesVersionNamed: self newerVersionString]
		ifNone: [^ self].
	
	(repo loadVersionFromFileNamed: self newerVersionString, '.mcz')
		merge.! !

!CtUpdateTask methodsFor: 'info' stamp: 'mt 4/9/2010 09:58:27.219'!
updateAvailable

	^ self newerVersionString notEmpty! !

Object subclass: #CtUpdater
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CT-Updater'!

!CtUpdater commentStamp: 'mt 4/9/2010 09:13:58.99' prior: 0!
I am used to update ProjectTalk conveniently.!

!CtUpdater class methodsFor: 'accessing' stamp: 'mt 4/9/2010 09:47:23.596'!
newPackages
	"Choose those packages that have only a local cache repository which implies that they are new."
	
	^ self pvtPackageNames
		collect: [:packageName | MCPackage named: packageName]
		thenSelect: [:package | 
			package workingCopy repositoryGroup repositories size = 1]! !

!CtUpdater class methodsFor: 'accessing' stamp: 'mt 4/9/2010 09:46:00.908'!
packages
	"Collect all packages that belong to ProjectTalk and that have repositories other than caches."

	^ self pvtPackageNames
		collect: [:packageName | MCPackage named: packageName]
		thenSelect: [:package | 
			(package workingCopy repositoryGroup repositories
				reject: [:repository | repository isKindOf: MCCacheRepository])
				notEmpty]! !

!CtUpdater class methodsFor: 'private' stamp: 'mt 4/9/2010 10:24:42.852'!
pvtPackageNames

	^ {
	'AST-Core'.
	'Refactoring-Core'.
	'Refactoring-Spelling'.
	'Shout'.
	'OmniBrowser'.
	'OB-Standard'.
	'OB-Morphic'.
	'OB-Refactory'.
	'OB-Shout'.
	'VB-Regex'.
	'SWAUtil'.
	'SWAOBExtensions'.
	'CodeTalk'}! !

!CtUpdater class methodsFor: 'updating' stamp: 'mt 4/9/2010 12:21:55.602'!
update

	| tasks msg |
	tasks := self packages
		collect: [:package |	CtUpdateTask package: package]
		thenSelect: [:task | task updateAvailable].
		
	tasks ifEmpty: [UserDialogBoxMorph
		inform: 'No updates available.'
		title: 'CodeTalk'.
		^ self].
	
	msg := tasks
		inject: 'Do you want to update?', String cr
		into: [:text :task | text, String cr, task currentVersionString, ' --> ', task newerVersionString].
		
	(UserDialogBoxMorph
		confirm: msg
		title: 'CodeTalk') ifTrue: [
			tasks
				do: [:task | task update]
				displayingProgress: [:task | 'Merging ', task newerVersionString, '...']].! !
